@@ -1,3 +1,5 @@
+// +build amd64,!windows arm64,!windows
+
 package machine
 
 import (
@@ -32,6 +34,7 @@ var (
 	ErrVMAlreadyExists                           = errors.New("VM already exists")
 	ErrVMAlreadyRunning                          = errors.New("VM already running")
 	ErrMultipleActiveVM                          = errors.New("only one VM can be active at a time")
+	ForwarderBinaryName                          = "gvproxy"
 )
 
 type Download struct {
@@ -55,10 +58,14 @@ type ListResponse struct {
 	LastUp    time.Time
 	Running   bool
 	VMType    string
+	CPUs      uint64
+	Memory    uint64
+	DiskSize  uint64
 }
 
 type SSHOptions struct {
-	Args []string
+	Username string
+	Args     []string
 }
 type StartOptions struct{}
 
